# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/08_pmlst.ipynb.

# %% auto 0
__all__ = ['process_pmlst_data', 'process_pmlst_data_from_cli']

# %% ../nbs/08_pmlst.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test
from . import core

# %% ../nbs/08_pmlst.ipynb 6
def process_pmlst_data(
    input_path: str,
    output_path: str = "./output.tsv",
    filter_columns: str = None,
    replace_header: str = None,
):
    """
    Command-line interface for processing PMLST data.

    This function sets up an argument parser to handle command-line arguments for processing PMLST data files.
    It supports specifying input and output file paths, replacing headers, filtering columns, and handling the presence or absence of headers in the input file.

    Arguments:
        input_path (str): Path to the input file.
        output_path (str): Path to the output file (default: './output.tsv').
        filter_columns (str): Columns to filter from the header (default: None).
        replace_header (str): New header to replace the existing one (default: None).
    """

    df = core.DataFrame()

    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input file {input_path} does not exist.")
    df.import_data(input_path, file_type="tsv")

    if replace_header:
        df.rename_header(replace_header)

    # df.show()

    df.export_data(output_path, file_type="tsv")


@call_parse
def process_pmlst_data_from_cli(
    input_path: str,
    output_path: str = "./output.tsv",
    filter_columns: str = None,
    replace_header: str = None,
):
    process_pmlst_data(input_path, output_path, filter_columns, replace_header)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_fastp.ipynb.

# %% auto 0
__all__ = ['process_fastp_data', 'process_fastp_data_from_cli']

# %% ../nbs/03_fastp.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test

#!export
from . import core

# %% ../nbs/03_fastp.ipynb 5
def process_fastp_data(
    input_path: str,
    output_path: str = "./output.tsv",
    replace_header: str = None,
    filter_columns: str = None,
):
    """
    Command-line interface for processing MLST data.

    This function sets up an argument parser to handle command-line arguments for processing FASTP data files.
    It supports specifying input and output file paths, replacing headers, filtering columns.

    Arguments:
        input_path (str): Path to the input file.
        output_path (str): Path to the output file (default: './output.tsv').
        replace_header (str): Header to replace the existing header (default: None).
        filter_columns (str): Columns to filter from the header (default: None).
    """

    df = core.DataFrame()
    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input file {input_path} does not exist.")
    df.import_data(input_path, file_type="json")

    if filter_columns:
        df.filter_columns(filter_columns)

    if replace_header:
        df.rename_header(replace_header)

    # df.show()

    df.export_data(output_path, file_type="tsv")


@call_parse
def process_fastp_data_from_cli(
    input_path: str,
    output_path: str = "./output.tsv",
    replace_header: str = None,
    filter_columns: str = None,
):
    process_fastp_data(input_path, output_path, replace_header, filter_columns)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/07_bracken.ipynb.

# %% auto 0
__all__ = ['process_bracken_data', 'process_bracken_data_from_cli']

# %% ../nbs/07_bracken.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test
import pandas as pd

#!export
from . import core

# %% ../nbs/07_bracken.ipynb 5
def process_bracken_data(
    input_path: str,
    output_path: str = "./output.tsv",
    add_header: str = "%ofreads, reads, notsure, rank, taxid, name",
    replace_header: str = None,
    filter_columns: str = None,
    transpose: bool = True,
):
    """
    Process Bracken data.

    This function processes Bracken data files by importing the data, optionally replacing the header,
    transposing the data, filtering columns, and exporting the processed data to an output file.

    Arguments:
        input_path (str): Path to the input file.
        output_path (str): Path to the output file (default: './output.tsv').
        replace_header (str): Header to replace the existing header (default: None).
        filter_columns (str): Columns to filter from the header (default: None).
        transpose (bool): Whether to transpose the data (default: True).
    """

    df = core.DataFrame()

    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input file {input_path} does not exist.")
    df.import_data(input_path, file_type="tsv", add_header=add_header)

    df.df = pd.concat(
        [
            df.df[df.df["rank"] == "U"],
            df.df[df.df["rank"] == "S"].sort_values(by="%ofreads", ascending=False),
        ]
    )
    df.df = df.df.head(3)

    df_output = core.DataFrame()
    df_output.df = pd.DataFrame(
        {
            "species1_unclassified_name": [df.df.iloc[1]["name"] + " + unclassified"],
            "species1_unclassified_pct": [
                df.df.iloc[1]["%ofreads"] + df.df.iloc[0]["%ofreads"]
            ],
            "species1_name": [df.df.iloc[1]["name"]],
            "species1_pct": [df.df.iloc[1]["%ofreads"]],
            "species2_name": [df.df.iloc[2]["name"]],
            "species2_pct": [df.df.iloc[2]["%ofreads"]],
            "unclassified_name": [df.df.iloc[0]["name"]],
            "unclassified_pct": [df.df.iloc[0]["%ofreads"]],
        }
    )

    if replace_header:
        df_output.rename_header(replace_header)

    if filter_columns:
        df_output.filter_columns(filter_columns)

    df_output.export_data(output_path, file_type="tsv")


@call_parse
def process_bracken_data_from_cli(
    input_path: str,
    output_path: str = "./output.tsv",
    add_header: str = "%ofreads, reads, notsure, rank, taxid, name",
    replace_header: str = None,
    filter_columns: str = None,
):
    process_bracken_data(input_path, output_path, replace_header, filter_columns)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['PACKAGE_NAME', 'DEV_MODE', 'PACKAGE_DIR', 'PROJECT_DIR', 'config', 'set_env_variables', 'get_config',
           'show_project_env_vars', 'get_samplesheet', 'hello_world', 'cli', 'DataFrame', 'import_nested_json_data',
           'export_nested_json_data', 'import_nested_xml_data', 'export_nested_xml_data', 'import_data',
           'rename_header', 'filter_columns', 'filter_rows', 'export_data', 'add_column', 'print_header', 'show']

# %% ../nbs/00_core.ipynb 4
# Need the bifrost_bridge for a few functions, this can be considered a static var

import importlib
import importlib.util
import os

PACKAGE_NAME: str = "bifrost_bridge"  # Make sure to adjust this to your package name
DEV_MODE: bool = (
    False  # set below to override, as this is in an export block it'll be exported while the dev mode section is not
)

PACKAGE_DIR = None
try:
    spec = importlib.util.find_spec(PACKAGE_NAME)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    PACKAGE_DIR = os.path.dirname(module.__file__)
except ImportError:
    DEV_MODE = True
except AttributeError:
    DEV_MODE = True
PROJECT_DIR = os.getcwd()  # override value in dev mode
if PROJECT_DIR.endswith("nbs"):
    DEV_MODE = True
    PROJECT_DIR = os.path.split(PROJECT_DIR)[0]

# %% ../nbs/00_core.ipynb 10
# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
import pandas  # For sample sheet manipulation
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script

# Project specific libraries

# %% ../nbs/00_core.ipynb 13
import importlib
import importlib.util


def set_env_variables(config_path: str, overide_env_vars: bool = True) -> bool:
    # Load dot env sets environmental values from a file, if the value already exists it will not be overwritten unless override is set to True.
    # If we have multiple .env files then we need to apply the one which we want to take precedence last with overide.

    # Order of precedence: .env file > environment variables > default values
    # When developing, making a change to the config will not be reflected until the environment is restarted

    # Set the env vars first, this is needed for the card.yaml to replace ENV variables
    # NOTE: You need to adjust PROJECT_NAME to your package name for this to work, the exception is only for dev purposes
    # This here checks if your package is installed, such as through pypi or through pip install -e  [.dev] for development. If it is then it'll go there and use the config files there as your default values.
    try:
        dotenv.load_dotenv(f"{PACKAGE_DIR}/config/config.default.env", override=False)
    except Exception as e:
        print(f"Error: {PACKAGE_DIR}/config/config.default.env does not exist")
        return False

    # 2. set values from file:
    if os.path.isfile(config_path):
        dotenv.load_dotenv(config_path, override=overide_env_vars)

    return True

# %% ../nbs/00_core.ipynb 15
import importlib
import importlib.util


def get_config(config_path: str = None, overide_env_vars: bool = True) -> dict:
    if config_path is None:
        config_path = ""
    # First sets environment with variables from config_path, then uses those variables to fill in appropriate values in the config.yaml file, the yaml file is then returned as a dict
    # If you want user env variables to take precedence over the config.yaml file then set overide_env_vars to False
    set_env_variables(config_path, overide_env_vars)

    config: dict = envyaml.EnvYAML(
        os.environ.get(
            "CORE_YAML_CONFIG_FILE", f"{PACKAGE_DIR}/config/config.default.yaml"
        ),
        strict=False,
    ).export()

    return config

# %% ../nbs/00_core.ipynb 17
# create a os.PathLike object
config = get_config(os.environ.get("CORE_CONFIG_FILE", ""))

# %% ../nbs/00_core.ipynb 19
def show_project_env_vars(config: dict) -> None:
    # Prints out all the project environment variables
    # This is useful for debugging and seeing what is being set
    for k, v in config.items():
        # If ENV var starts with PROJECTNAME_ then print
        if k.startswith(config["CORE_PROJECT_VARIABLE_PREFIX"]):
            print(f"{k}={v}")

# %% ../nbs/00_core.ipynb 22
import pandas as pd


def get_samplesheet(sample_sheet_config: dict) -> pd.DataFrame:
    # Load the sample sheet into a pandas dataframe
    # If columns is not None then it will only load those columns
    # If the sample sheet is a csv then it will load it as a csv, otherwise it will assume it's a tsv

    # Expected sample_sheet_config:
    # sample_sheet:
    #   path: path/to/sample_sheet.tsv
    #   delimiter: '\t' # Optional, will assume , for csv and \t otherwises
    #   header: 0 # Optional, 0 indicates first row is header, None indicates no header
    #   columns: ['column1', 'column2', 'column3'] # Optional, if not provided all columns will be used

    # Example sample sheet:
    # #sample_id	file_path	metadata1	metadata2
    # Sample1	/path/to/sample1.fasta	value1	option1
    # Sample2	/path/to/sample2.fasta	value2	option2
    # Sample3	/path/to/sample3.fasta	value3	option1
    # Sample4	/path/to/sample4.fasta	value1	option2
    # Sample5	/path/to/sample5.fasta	value2	option1

    # This function should also handle ensuring the sample sheet is in the correct format, such as ensuring the columns are correct and that the sample names are unique.
    if not os.path.isfile(sample_sheet_config["path"]):
        raise FileNotFoundError(f"File {sample_sheet_config['path']} does not exist")
    if "delimiter" in sample_sheet_config:
        delimiter = sample_sheet_config["delimiter"]
    else:
        # do a best guess based on file extension
        delimiter = "," if sample_sheet_config["path"].endswith(".csv") else "\t"
    header = 0
    # if "header" in sample_sheet_config:
    #     header = sample_sheet_config["header"]
    # else:
    #     # check if the first line starts with a #, if so lets assume it's a header otherwise assume there isn't one
    #     with open(sample_sheet_config["path"], "r") as f:
    #         first_line = f.readline()
    #         header = 0 if first_line.startswith("#") else None
    if "columns" in sample_sheet_config:
        columns = sample_sheet_config[
            "columns"
        ]  # note the # for the first item needs to be stripped to compare to the columns
    else:
        columns = None  # implies all columns
    try:
        # note when we have a header the first column may begin with a #, so we need to remove it
        df = pd.read_csv(
            sample_sheet_config["path"],
            delimiter=delimiter,
            header=header,
            comment=None,
        )
    except Exception as e:
        print(
            "Error: Could not load sample sheet into dataframe, you have a problem with your sample sheet or the configuration."
        )
        raise e

    # Check the first header has a # in it, if so remove it for only that item
    if df.columns[0].startswith("#"):
        df.columns = [col.lstrip("#") for col in df.columns]
    # Ensure the sample sheet has the correct columns
    if columns is not None and not all([col in df.columns for col in columns]):
        raise ValueError("Error: Sample sheet does not have the correct columns")
    # also drop columns which are not needed
    if columns is not None:
        df = df[columns]

    # Clean the df of any extra rows that can be caused by empty lines in the sample sheet
    df = df.dropna(how="all")
    return df

# %% ../nbs/00_core.ipynb 24
def hello_world(name: str = "Not given") -> str:
    return f"Hello World! My name is {name}"

# %% ../nbs/00_core.ipynb 28
from fastcore.script import call_parse


@call_parse
def cli(
    name: str,  # Your name
    config_file: str = None,  # config file to set env vars from
):
    """
    This will print Hello World! with your name
    """
    config = get_config(config_file)  # Set env vars and get config variables
    if name is not None:
        config["example"]["input"]["name"] = name

    print(hello_world(config["example"]["input"]["name"]))

# %% ../nbs/00_core.ipynb 32
import json
import pandas as pd
import json
import yaml
import xml.etree.ElementTree as ET
from xml.dom import minidom

# %% ../nbs/00_core.ipynb 34
class DataFrame:
    def __init__(self, data=None):
        """
        Initialize the DataFrame object.
        :param data: Optional initial data for the DataFrame.
        """
        if data is not None:
            self.df = pd.DataFrame(data)
        else:
            self.df = pd.DataFrame()

# %% ../nbs/00_core.ipynb 36
def import_nested_json_data(self, json_file_path):
    """
    Import nested JSON data from a file and create headers by combining headers with underscores.
    :param json_file_path: Path to the JSON file.
    """

    with open(json_file_path, "r") as file:
        nested_json = json.load(file)

    def flatten_json(y):
        out = {}

        def flatten(x, name=""):
            if type(x) is dict:
                for a in x:
                    flatten(x[a], name + a + "£")
            elif type(x) is list:
                i = 1  # Start numbering from 1
                for a in x:
                    flatten(a, name + str(i) + "£")
                    i += 1
            else:
                out[name[:-1]] = x

        flatten(y)
        return out

    if isinstance(nested_json, list):
        flat_data = [flatten_json(item) for item in nested_json]
        self.df = pd.json_normalize(flat_data)
    else:
        flat_data = flatten_json(nested_json)
        self.df = pd.json_normalize(flat_data)


def export_nested_json_data(self, json_file_path):
    """
    Export the DataFrame to a nested JSON file by unraveling headers with underscores.
    :param json_file_path: Path to the JSON file.
    """

    def unflatten_json(flat_dict):
        out = {}

        for key, value in flat_dict.items():
            keys = key.split("£")
            d = out
            for k in keys[:-1]:
                d = d.setdefault(k, {})
            d[keys[-1]] = value
        return out

    nested_json_list = [
        unflatten_json(row) for row in self.df.to_dict(orient="records")
    ]

    def convert_to_list(d):
        for key, value in d.items():
            if isinstance(value, dict):
                if all(k.isdigit() for k in value.keys()):
                    d[key] = [
                        v
                        for k, v in sorted(value.items(), key=lambda item: int(item[0]))
                    ]
                else:
                    convert_to_list(value)
            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict):
                        convert_to_list(item)

    def has_more_layers(d):
        for key, value in d.items():
            if isinstance(value, dict):
                if any(k.isdigit() for k in value.keys()):
                    return True
                else:
                    if has_more_layers(value):
                        return True
            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict) and has_more_layers(item):
                        return True
        return False

    for i, nested_json in enumerate(nested_json_list):
        while has_more_layers(nested_json):
            convert_to_list(nested_json)

        if all(key.split("£")[0].isdigit() for key in nested_json.keys()):
            nested_json_list[i] = [
                nested_json[str(i)] for i in range(1, len(nested_json) + 1)
            ]

    if len(nested_json_list) == 1:
        nested_json_list = nested_json_list[0]

    with open(json_file_path, "w") as file:
        json.dump(nested_json_list, file, indent=4)


DataFrame.import_nested_json_data = import_nested_json_data
DataFrame.export_nested_json_data = export_nested_json_data

# %% ../nbs/00_core.ipynb 38
def import_nested_xml_data(self, xml_file_path):
    """
    Import nested XML data from a file and create headers by combining headers with underscores.
    :param xml_file_path: Path to the XML file.
    """
    tree = ET.parse(xml_file_path)
    root = tree.getroot()

    def flatten_xml(element, parent_name=""):
        items = {}
        for child in element:
            child_name = f"{parent_name}{element.tag}_{child.tag}_"
            if len(child):
                items.update(flatten_xml(child, child_name))
            else:
                items[child_name[:-1]] = child.text
        return items

    flat_data = [flatten_xml(child) for child in root]
    self.df = pd.json_normalize(flat_data)


def export_nested_xml_data(self, xml_file_path):
    root = ET.Element("Invetory")
    for _, row in self.df.iterrows():
        tag_created = False
        for col in self.df.columns:
            tags = col.split("_")
            for tag in tags[:-1]:
                # print(tags[:-1])
                if tag_created is False:
                    new_tag = ET.SubElement(root, tag)
                    tag_created = True
                ET.SubElement(new_tag, tags[-1]).text = str(row[col])

    # Convert to string and pretty print using minidom
    xml_str = ET.tostring(root, encoding="utf-8")
    parsed_str = minidom.parseString(xml_str)
    pretty_xml_str = parsed_str.toprettyxml(indent="  ")

    with open(xml_file_path, "w", encoding="utf-8") as f:
        f.write(pretty_xml_str)


DataFrame.import_nested_xml_data = import_nested_xml_data
DataFrame.export_nested_xml_data = export_nested_xml_data

# %% ../nbs/00_core.ipynb 40
def import_data(self, file_path, file_type="csv", add_header=""):
    """
    Import data from a CSV, TSV, JSON, XML, or YAML file.
    :param file_path: Path to the file.
    :param file_type: Type of the file ('csv', 'tsv', 'json', 'xml', 'yaml').
    :param delimiter: Delimiter used in the file (default is comma for CSV).
    """

    # Check if add_header is a string and split it into a list
    if isinstance(add_header, str):
        if len(add_header) > 0:
            add_header = add_header.replace(" ", "").split(",")

    if file_type == "csv":
        self.df = pd.read_csv(
            file_path, delimiter=",", header=None if add_header else 0, index_col=False
        )
        if add_header:
            if len(add_header) != len(self.df.columns):
                raise ValueError(
                    f"Error: Number of new column names ({len(add_header)}) must match the number of columns in the DataFrame ({len(self.df.columns)})."
                )
            else:
                self.df.columns = add_header

    elif file_type == "tsv":
        self.df = pd.read_csv(
            file_path, delimiter="\t", header=None if add_header else 0, index_col=False
        )
        if add_header:
            if len(add_header) != len(self.df.columns):
                raise ValueError(
                    f"Error: Number of new column names ({len(add_header)}) must match the number of columns in the DataFrame ({len(self.df.columns)})."
                )
            else:
                self.df.columns = add_header

    elif file_type == "json":
        self.import_nested_json_data(file_path)
    elif file_type == "yaml":
        with open(file_path, "r") as f:
            data = yaml.safe_load(f)
            self.df = pd.json_normalize(data)
    elif file_type == "xml":
        self.import_nested_xml_data(file_path)
    #    tree = ET.parse(file_path)
    #    root = tree.getroot()
    #    data = self._xml_to_dict(root)
    #    self.df = pd.json_normalize(data)
    #    self.df.columns = [col.replace('item.', '') for col in self.df.columns]
    # else:
    #    raise ValueError("Unsupported file type. Supported types: 'csv', 'tsv', 'json', 'xml', 'yaml'.")


def rename_header(self, new_columns):
    """
    Rename columns in the DataFrame.
    :param new_columns: Comma-separated string of new column names.
    """
    new_columns_list = [col.strip() for col in new_columns.split(",")]
    if len(new_columns_list) != len(self.df.columns):
        print(
            "Error: Number of new column names must match the number of columns in the DataFrame."
        )
        print("Current header:", self.df.columns.tolist())
    else:
        self.df.columns = new_columns_list


def filter_columns(self, columns):
    """
    Filter the DataFrame to include only specified columns.
    :param columns: List of columns to include or list of boolean values.
    """
    if all(isinstance(col, str) for col in columns):
        # Case 1: Comma-separated string of column names
        columns_list = [col.strip() for col in columns.split(",")]
        if not all(col in self.df.columns for col in columns_list):
            missing_cols = [col for col in columns_list if col not in self.df.columns]
            raise ValueError(
                f"Error: The following columns do not exist in the DataFrame: {missing_cols}"
            )
        self.df = self.df[columns_list]
    elif all(isinstance(col, bool) for col in columns):
        # Case 2: List of boolean values
        if len(columns) != len(self.df.columns):
            raise ValueError(
                "Error: Number of boolean values must match the number of columns in the DataFrame."
            )
        self.df = self.df.loc[:, columns]
    else:
        raise ValueError(
            "Error: columns parameter must be a list of column names or a list of boolean values."
        )


def filter_rows(self, condition):
    """
    Filter the DataFrame to include only rows that meet the condition.
    :param condition: List of integers (row indices starting with 1) or list of boolean values.
    """
    if all(isinstance(cond, bool) for cond in condition):
        # Case 1: List of boolean values
        if len(condition) != len(self.df):
            raise ValueError(
                "Error: Number of boolean values must match the number of rows in the DataFrame."
            )
        self.df = self.df[condition]
    elif all(isinstance(cond, int) for cond in condition):
        # Case 2: List of integers (row indices starting with 1)
        if any(cond < 1 or cond > len(self.df) for cond in condition):
            raise ValueError(
                "Error: One or more row indices are outside the scope of the DataFrame."
            )
        self.df = self.df.iloc[[cond - 1 for cond in condition]]
    else:
        raise ValueError(
            "Error: condition parameter must be a list of integers or a list of boolean values."
        )
    # Renumber the rows starting from 1
    self.df.index = range(1, len(self.df) + 1)


def export_data(self, file_path, file_type="csv"):
    """
    Export data to a CSV, TSV, JSON, YAML, or XML file.
    :param file_path: Path to the file.
    :param file_type: Type of the file ('csv', 'tsv', 'json', 'yaml', 'xml').
    :param delimiter: Delimiter to use in the file (default is comma for CSV/TSV).
    """
    if file_type == "csv":
        self.df.to_csv(file_path, index=False, sep=",")
    elif file_type == "tsv":
        self.df.to_csv(file_path, index=False, sep="\t")
    elif file_type == "json":
        self.export_nested_json_data(file_path)
    elif file_type == "yaml":
        with open(file_path, "w") as f:
            yaml.dump(self.df.to_dict(orient="records"), f, sort_keys=False)
    elif file_type == "xml":
        self.export_nested_xml_data(file_path)
    else:
        raise ValueError(
            "Unsupported file type. Supported types: 'csv', 'tsv', 'json', 'yaml', 'xml'."
        )


def add_column(self, column_name, data):
    """
    Add a new column to the DataFrame.
    :param column_name: Name of the new column.
    :param data: Data for the new column (must match the number of rows).
    """
    if column_name in self.df.columns:
        raise ValueError(
            f"Error: Column '{column_name}' already exists in the DataFrame."
        )
    if len(self.df) == 0:
        # If DataFrame is empty, create it with the new column
        self.df = pd.DataFrame({column_name: data})
    else:
        if len(data) != len(self.df):
            raise ValueError(
                "Error: Length of data must match the number of rows in the DataFrame."
            )
        self.df[column_name] = data


def print_header(self):
    """
    Print the header of the DataFrame as a list.
    """
    print(self.df.columns.tolist())


def show(self):
    """
    Display the DataFrame.
    """
    print(self.df)


DataFrame.import_data = import_data
DataFrame.rename_header = rename_header
DataFrame.filter_columns = filter_columns
DataFrame.filter_rows = filter_rows
DataFrame.export_data = export_data
DataFrame.print_header = print_header
DataFrame.add_column = add_column
DataFrame.show = show



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/bifrost_bridge',
                'doc_host': 'https://$GIT_USER_NAME.github.io',
                'git_url': 'https://github.com/$GIT_USER_NAME/bifrost_bridge',
                'lib_path': 'bifrost_bridge'},
  'syms': { 'bifrost_bridge.amrfinderplus': { 'bifrost_bridge.amrfinderplus.process_amrfinderplus_data': ( 'amrfinderplus.html#process_amrfinderplus_data',
                                                                                                           'bifrost_bridge/amrfinderplus.py'),
                                              'bifrost_bridge.amrfinderplus.process_amrfinderplus_data_from_cli': ( 'amrfinderplus.html#process_amrfinderplus_data_from_cli',
                                                                                                                    'bifrost_bridge/amrfinderplus.py')},
            'bifrost_bridge.bifrost': { 'bifrost_bridge.bifrost.process_qc_data': ( 'bifrost.html#process_qc_data',
                                                                                    'bifrost_bridge/bifrost.py')},
            'bifrost_bridge.bracken': { 'bifrost_bridge.bracken.process_bracken_data': ( 'bracken.html#process_bracken_data',
                                                                                         'bifrost_bridge/bracken.py'),
                                        'bifrost_bridge.bracken.process_bracken_data_from_cli': ( 'bracken.html#process_bracken_data_from_cli',
                                                                                                  'bifrost_bridge/bracken.py')},
            'bifrost_bridge.cli': {'bifrost_bridge.cli.cli': ('cli.html#cli', 'bifrost_bridge/cli.py')},
            'bifrost_bridge.core': { 'bifrost_bridge.core.DataFrame': ('core.html#dataframe', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.DataFrame.__init__': ('core.html#dataframe.__init__', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.add_column': ('core.html#add_column', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.cli': ('core.html#cli', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.export_data': ('core.html#export_data', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.export_nested_json_data': ( 'core.html#export_nested_json_data',
                                                                                      'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.export_nested_xml_data': ( 'core.html#export_nested_xml_data',
                                                                                     'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.filter_columns': ('core.html#filter_columns', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.filter_rows': ('core.html#filter_rows', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.get_config': ('core.html#get_config', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.get_samplesheet': ('core.html#get_samplesheet', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.hello_world': ('core.html#hello_world', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.import_data': ('core.html#import_data', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.import_nested_json_data': ( 'core.html#import_nested_json_data',
                                                                                      'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.import_nested_xml_data': ( 'core.html#import_nested_xml_data',
                                                                                     'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.print_header': ('core.html#print_header', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.rename_header': ('core.html#rename_header', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.set_env_variables': ('core.html#set_env_variables', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.show': ('core.html#show', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.show_project_env_vars': ( 'core.html#show_project_env_vars',
                                                                                    'bifrost_bridge/core.py')},
            'bifrost_bridge.fastp': { 'bifrost_bridge.fastp.process_fastp_data': ( 'fastp.html#process_fastp_data',
                                                                                   'bifrost_bridge/fastp.py'),
                                      'bifrost_bridge.fastp.process_fastp_data_from_cli': ( 'fastp.html#process_fastp_data_from_cli',
                                                                                            'bifrost_bridge/fastp.py')},
            'bifrost_bridge.mlst': { 'bifrost_bridge.mlst.check_if_mlst_empty': ('mlst.html#check_if_mlst_empty', 'bifrost_bridge/mlst.py'),
                                     'bifrost_bridge.mlst.process_mlst_data': ('mlst.html#process_mlst_data', 'bifrost_bridge/mlst.py'),
                                     'bifrost_bridge.mlst.process_mlst_data_from_cli': ( 'mlst.html#process_mlst_data_from_cli',
                                                                                         'bifrost_bridge/mlst.py')},
            'bifrost_bridge.plasmidfinder': { 'bifrost_bridge.plasmidfinder.process_plasmidfinder_data': ( 'plasmidfinder.html#process_plasmidfinder_data',
                                                                                                           'bifrost_bridge/plasmidfinder.py'),
                                              'bifrost_bridge.plasmidfinder.process_plasmidfinder_data_from_cli': ( 'plasmidfinder.html#process_plasmidfinder_data_from_cli',
                                                                                                                    'bifrost_bridge/plasmidfinder.py')},
            'bifrost_bridge.pmlst': { 'bifrost_bridge.pmlst.process_pmlst_data': ( 'pmlst.html#process_pmlst_data',
                                                                                   'bifrost_bridge/pmlst.py'),
                                      'bifrost_bridge.pmlst.process_pmlst_data_from_cli': ( 'pmlst.html#process_pmlst_data_from_cli',
                                                                                            'bifrost_bridge/pmlst.py')},
            'bifrost_bridge.quast': { 'bifrost_bridge.quast.process_quast_data': ( 'quast.html#process_quast_data',
                                                                                   'bifrost_bridge/quast.py'),
                                      'bifrost_bridge.quast.process_quast_data_from_cli': ( 'quast.html#process_quast_data_from_cli',
                                                                                            'bifrost_bridge/quast.py')},
            'bifrost_bridge.rmlst': { 'bifrost_bridge.rmlst.process_rmlst_data': ( 'rmlst.html#process_rmlst_data',
                                                                                   'bifrost_bridge/rmlst.py'),
                                      'bifrost_bridge.rmlst.process_rmlst_data_from_cli': ( 'rmlst.html#process_rmlst_data_from_cli',
                                                                                            'bifrost_bridge/rmlst.py')},
            'bifrost_bridge.shovill': { 'bifrost_bridge.shovill.process_shovill_data': ( 'shovill.html#process_shovill_data',
                                                                                         'bifrost_bridge/shovill.py'),
                                        'bifrost_bridge.shovill.process_shovill_data_from_cli': ( 'shovill.html#process_shovill_data_from_cli',
                                                                                                  'bifrost_bridge/shovill.py')},
            'bifrost_bridge.ssiamb': { 'bifrost_bridge.ssiamb.process_ssiamb_data': ( 'ssiamb.html#process_ssiamb_data',
                                                                                      'bifrost_bridge/ssiamb.py'),
                                       'bifrost_bridge.ssiamb.process_ssiamb_data_from_cli': ( 'ssiamb.html#process_ssiamb_data_from_cli',
                                                                                               'bifrost_bridge/ssiamb.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_shovill.ipynb.

# %% auto 0
__all__ = ['process_shovill_data', 'process_shovill_data_from_cli']

# %% ../nbs/10_shovill.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test
from . import core

# %% ../nbs/10_shovill.ipynb 6
def process_shovill_data(
    input_path: str, output_path: str = "./output.tsv", average_coverage: float = True
):
    """
    Command-line interface for processing Shovill data.

    This function sets up an argument parser to handle command-line arguments for processing Shovill log output files.
    It supports specifying input and output file paths.

    Arguments:
        input_path (str): Path to the input file.
        output_path (str): Path to the output file (default: './output.tsv').
        average_coverage (float): Whether to calculate average coverage (default: True).
    """

    df = core.DataFrame()

    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input file {input_path} does not exist.")

    with open(input_path, "r") as f:
        for line in f:
            match = re.search(r"Average coverage is ([\d\.]+) and alpha is", line)
            if match:
                avg_coverage = float(match.group(1))
                break

    data_frame_columns = ["Average_Coverage"]
    # Add columns to DataFrame
    if average_coverage:
        if avg_coverage is None:
            df.add_column("Average_Coverage", [None])
        else:
            df.add_column("Average_Coverage", [avg_coverage])

    # df.show()

    df.export_data(output_path, file_type="tsv")


@call_parse
def process_shovill_data_from_cli(
    input_path: str, output_path: str = "./output.tsv", average_coverage: float = True
):
    process_shovill_data(input_path, output_path, average_coverage)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/99_bifrost.ipynb.

# %% auto 0
__all__ = ['process_qc_data']

# %% ../nbs/99_bifrost.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test

#!export
from . import core

# %% ../nbs/99_bifrost.ipynb 5
from .mlst import process_mlst_data
from .fastp import process_fastp_data
from .quast import process_quast_data
from .plasmidfinder import process_plasmidfinder_data
from .amrfinderplus import process_amrfinderplus_data
from .bracken import process_bracken_data
from .pmlst import process_pmlst_data
from .rmlst import process_rmlst_data
from .shovill import process_shovill_data
from .ssiamb import process_ssiamb_data
import pandas as pd


@call_parse
def process_qc_data(
    mlst_path: str = None,
    fastp_path: str = None,
    quast_path: str = None,
    plasmidfinder_path: str = None,
    bracken_path: str = None,
    amrfinder_path: str = None,
    pmlst_path: str = None,
    rmlst_path: str = None,
    shovill_path: str = None,
    ssiamb_path: str = None,
    combine_output: bool = True,
    output_path: str = "./output.tsv",
):
    """
    Command-line interface for processing QC data.

    This function processes MLST, FASTP, QUAST, PlasmidFinder, and Bracken data files based on the provided command-line arguments.
    It supports specifying input file paths for MLST, FASTP, QUAST, PlasmidFinder, and Bracken data, and outputs the processed data to specified paths.

    Arguments:
        mlst_path (str): Path to the MLST input file.
        fastp_path (str): Path to the FASTP input file.
        quast_path (str): Path to the QUAST input file.
        plasmidfinder_path (str): Path to the PlasmidFinder input file.
        bracken_path (str): Path to the Bracken input file.
        amrfinder_path (str): Path to the AMRFinder input file.
        pmlst_path (str): Path to the PMLST input file.
        shovill_path (str): Path to the Shovill input file.
        output_path (str): Path to the output file (default: './output.tsv').
    """
    if mlst_path is not None:
        if not os.path.exists(mlst_path):
            raise FileNotFoundError(f"File not found: {mlst_path}")
        process_mlst_data(
            input_path=mlst_path,
            output_path="parsed_mlst.tsv",
            replace_header=None,
            # filter_columns="SampleID, Species, ST",
            remove_sampleid=True,
            add_header="SampleID, MLST_Species, MLST_ST, MLST_Alleles",
            combine_alleles=True,
        )

    if fastp_path is not None:
        if not os.path.exists(fastp_path):
            raise FileNotFoundError(f"File not found: {fastp_path}")
        process_fastp_data(
            input_path=fastp_path,
            output_path="parsed_fastp.tsv",
            filter_columns="summary£before_filtering£total_reads, summary£before_filtering£read1_mean_length, summary£before_filtering£read2_mean_length, summary£after_filtering£total_reads, summary£after_filtering£read1_mean_length, summary£after_filtering£read2_mean_length, filtering_result£low_quality_reads, filtering_result£too_many_N_reads, filtering_result£too_short_reads, filtering_result£too_long_reads, duplication£rate, adapter_cutting£adapter_trimmed_reads, adapter_cutting£adapter_trimmed_bases, read1_before_filtering£total_cycles, read1_after_filtering£total_cycles, read2_before_filtering£total_cycles, read2_after_filtering£total_cycles",
            replace_header="fastp_Before_Filtering_Total_Reads,fastp_Before_Filtering_Read1_Mean_Length,fastp_Before_Filtering_Read2_Mean_Length,fastp_After_Filtering_Total_Reads,fastp_After_Filtering_Read1_Mean_Length,fastp_After_Filtering_Read2_Mean_Length,fastp_Low_Quality_Reads,fastp_Too_Many_N_Reads,fastp_Too_Short_Reads,fastp_Too_Long_Reads,fastp_Duplication_Rate,fastp_Adapter_Trimmed_Reads,fastp_Adapter_Trimmed_Bases,fastp_Read1_Before_Filtering_Total_Cycles,fastp_Read1_After_Filtering_Total_Cycles,fastp_Read2_Before_Filtering_Total_Cycles,fastp_Read2_After_Filtering_Total_Cycles",
        )

    if quast_path is not None:
        if not os.path.exists(quast_path):
            raise FileNotFoundError(f"File not found: {quast_path}")
        process_quast_data(
            input_path=quast_path,
            output_path="parsed_quast.tsv",
            filter_columns="# contigs, Largest contig, Total length, GC (%), N50, N90, L50, L90",
            replace_header="Quast_Contigs,Quast_Largest_Contig,Quast_Total_Length,Quast_GC_Pct,Quast_N50,Quast_N90,Quast_L50,Quast_L90",
            transpose=True,
        )

    if plasmidfinder_path is not None:
        if not os.path.exists(plasmidfinder_path):
            raise FileNotFoundError(f"File not found: {plasmidfinder_path}")
        process_plasmidfinder_data(
            input_path=plasmidfinder_path,
            output_path="parsed_plasmidfinder.tsv",
            filter_columns="Database,Plasmid,Identity,Query / Template length,Contig",
            replace_header="PFInder_Database,PFinder_Plasmid,PFinder_Identity,PFinder_Coverage,PFinder_Contig",
            convert_coverage=True,
            filter_contig=True,
        )

    if bracken_path is not None:
        if not os.path.exists(bracken_path):
            raise FileNotFoundError(f"File not found: {bracken_path}")
        process_bracken_data(
            input_path=bracken_path,
            output_path="parsed_bracken.tsv",
            replace_header="Bracken_Species,Bracken_Species_Pct,Bracken_Species1,Bracken_Species1_Pct,Bracken_Species2,Bracken_Species2_Pct,Bracken_Unclassified,Bracken_Unclassified_Pct",
        )

    if amrfinder_path is not None:
        if not os.path.exists(amrfinder_path):
            raise FileNotFoundError(f"File not found: {amrfinder_path}")
        process_amrfinderplus_data(
            input_path=amrfinder_path,
            output_path="parsed_amrfinder.tsv",
            filter_columns="Contig id,Start,Stop,Strand,Gene symbol,Sequence name,Subclass,% Coverage of reference sequence,% Identity to reference sequence",
            replace_header="AMR_ContigID,AMR_Start,AMR_Stop,AMR_Strand,AMR_ElementSymbol,AMR_ElementName,AMR_Subclass,AMR_Coverage,AMR_Identity",
        )

    if pmlst_path is not None:
        if not os.path.exists(pmlst_path):
            raise FileNotFoundError(f"File not found: {pmlst_path}")
        process_pmlst_data(
            input_path=pmlst_path,
            output_path="parsed_pmlst.tsv",
            replace_header="pMLST_plasmids,pMLST_IncF,pMLST_IncI1,pMLST_IncA/C,pMLST_IncHI1,pMLST_IncHI2,pMLST_IncN,pMLST_summary",
        )

    if rmlst_path is not None:
        if not os.path.exists(rmlst_path):
            raise FileNotFoundError(f"File not found: {rmlst_path}")
        process_rmlst_data(
            input_path=rmlst_path,
            output_path="parsed_rmlst.tsv",
            filter_columns="taxon,rank,support",
            replace_header="rMLST_match,rMLST_rank,rMLST_support",
        )

    if shovill_path is not None:
        if not os.path.exists(shovill_path):
            raise FileNotFoundError(f"File not found: {shovill_path}")
        process_shovill_data(
            input_path=shovill_path,
            output_path="parsed_shovill.tsv",
            average_coverage=True,
        )

    if ssiamb_path is not None:
        if not os.path.exists(ssiamb_path):
            raise FileNotFoundError(f"File not found: {ssiamb_path}")
        process_ssiamb_data(
            input_path=ssiamb_path,
            output_path="parsed_ssiamb.tsv",
            replace_header="ssiamb_count",
            filter_columns="ambiguous_snv_count",
        )

    if combine_output:
        # List of output files that were actually created
        output_files = []
        if mlst_path is not None:
            if os.path.getsize("parsed_mlst.tsv") > 0:
                output_files.append("parsed_mlst.tsv")
        if fastp_path is not None:
            output_files.append("parsed_fastp.tsv")
        if quast_path is not None:
            output_files.append("parsed_quast.tsv")
        if plasmidfinder_path is not None:
            output_files.append("parsed_plasmidfinder.tsv")
        if amrfinder_path is not None:
            output_files.append("parsed_amrfinder.tsv")
        if bracken_path is not None:
            output_files.append("parsed_bracken.tsv")
        if pmlst_path is not None:
            output_files.append("parsed_pmlst.tsv")
        if rmlst_path is not None:
            output_files.append("parsed_rmlst.tsv")
        if shovill_path is not None:
            output_files.append("parsed_shovill.tsv")
        if ssiamb_path is not None:
            output_files.append("parsed_ssiamb.tsv")

        # Read and concatenate all output files
        combined_df = pd.concat(
            [pd.read_csv(file, sep="\t") for file in output_files], axis=1
        )

        # Save the combined dataframe to the specified output path
        combined_df.to_csv(output_path, sep="\t", index=False)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_mlst.ipynb.

# %% auto 0
__all__ = ['process_mlst_data', 'check_if_mlst_empty', 'process_mlst_data_from_cli']

# %% ../nbs/02_mlst.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test
from . import core

# %% ../nbs/02_mlst.ipynb 6
def process_mlst_data(
    input_path: str,
    output_path: str = "./output.tsv",
    add_header: str = None,
    replace_header: str = None,
    filter_columns: str = None,
    remove_sampleid: bool = False,
    combine_alleles: bool = False,
):
    """
    Command-line interface for processing MLST data.

    This function sets up an argument parser to handle command-line arguments for processing MLST data files.
    It supports specifying input and output file paths, replacing headers, filtering columns, and handling the presence or absence of headers in the input file.

    Arguments:
        input_path (str): Path to the input file.
        output_path (str): Path to the output file (default: './output.tsv').
        add_header (str): Header to add if the header does not exist in the input file (default: None).
        replace_header (str): Header to replace the existing header (default: None).
        filter_columns (str): Columns to filter from the header (default: None).
        remove_sampleid (bool): Whether to remove the SampleID column (default: False).
        combine_alleles (bool): Whether to combine allele columns into one (default: False).
    """

    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input file {input_path} does not exist.")

    df_check = core.DataFrame()
    df_check.import_data(input_path, file_type="tsv")

    if check_if_mlst_empty(df_check):
        with open(output_path, "w") as f:
            f.write("")
        return

    df = core.DataFrame()
    df.import_data(input_path, file_type="tsv")

    if combine_alleles:
        # print 4th and further values of df.df.columns in a comma separated list
        combine_alleles_string = ",".join(df.df.columns[3:])

        # Replace 4th column with combined string
        cols = list(df.df.columns)
        cols[3] = combine_alleles_string
        df.df.columns = cols

        # Remove columns after 4th
        df.df = df.df.iloc[:, :4]

    if add_header:
        header_list = add_header.split(", ")
        # Add more headers if not combining alleles
        if not combine_alleles:
            header_list[3] = header_list[3] + "1"
            for i in range(4, len(df.df.columns)):
                header_list.append(header_list[3][:-1] + str(i - 2))
        df.df.loc[-1] = df.df.columns  # adding header as first row
        df.df.index = df.df.index + 1  # shifting index
        df.df = df.df.sort_index()  # sorting by index to move the header row to the top
        df.df.columns = header_list  # setting new header

    if replace_header:
        df.rename_header(replace_header)

    if filter_columns:
        df.filter_columns(filter_columns)

    if remove_sampleid:
        # Remove the first column (SampleID) if it exists
        if df.df.columns[0] == "SampleID":
            df.df = df.df.iloc[:, 1:]

    df.export_data(output_path, file_type="tsv")


def check_if_mlst_empty(df):
    if df.df.columns[1] == "-":
        return True
    else:
        return False


@call_parse
def process_mlst_data_from_cli(
    input_path: str,
    output_path: str = "./output.tsv",
    add_header: str = None,
    replace_header: str = None,
    filter_columns: str = None,
    remove_sampleid: bool = False,
    combine_alleles: bool = False,
):
    process_mlst_data(
        input_path,
        output_path,
        add_header,
        replace_header,
        filter_columns,
        remove_sampleid,
        combine_alleles,
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_plasmidfinder.ipynb.

# %% auto 0
__all__ = ['process_plasmidfinder_data', 'process_plasmidfinder_data_from_cli']

# %% ../nbs/05_plasmidfinder.ipynb 2
# That export there, it makes sure this code goes into the module.

# standard libs
import os
import re

# Common to template
# add into settings.ini, requirements, package name is python-dotenv, for conda build ensure `conda config --add channels conda-forge`
import dotenv  # for loading config from .env files, https://pypi.org/project/python-dotenv/
import envyaml  # Allows to loads env vars into a yaml file, https://github.com/thesimj/envyaml
import fastcore  # To add functionality related to nbdev development, https://github.com/fastai/fastcore/
from fastcore import (
    test,
)
from fastcore.script import (
    call_parse,
)  # for @call_parse, https://fastcore.fast.ai/script
import json  # for nicely printing json and yaml
from fastcore import test
from . import core

# %% ../nbs/05_plasmidfinder.ipynb 6
def process_plasmidfinder_data(
    input_path: str,
    output_path: str = "./output.tsv",
    replace_header: str = None,
    filter_columns: str = None,
    add_header: str = None,
    convert_coverage: bool = False,
    filter_contig: bool = False,
):
    """
    Command-line interface for processing plasmidfinder data.

    This function sets up an argument parser to handle command-line arguments for processing plasmidfinder data files.
    It supports specifying input and output file paths, replacing headers, filtering columns, and handling the presence or absence of headers in the input file.

    Arguments:
        input_path (str): Path to the input file.
        output_path (str): Path to the output file (default: './output.tsv').
        replace_header (str): Header to replace the existing header (default: None).
        filter_columns (str): Columns to filter from the header (default: None).
        header_exists (int): Indicates if the header exists in the input file (default: 1).
        add_header (str): Header to add if the header does not exist in the input file (default: None).
        convert_coverage (bool): If True, converts coverage values in the 'Query / Template length' column to percentages (default: False).
        filter_contig (bool): If True, filters out 'Contig' column to just contig number (default: False).
    """

    df = core.DataFrame()

    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input file {input_path} does not exist.")
    df.import_data(input_path, file_type="tsv", add_header=add_header)

    def concatenate_vector(x, sep=","):
        return ",".join([str(i) for i in x])

    df_agg = df.df.apply(concatenate_vector, axis=0)
    df.df = df_agg.to_frame().T

    # PFinder_Coverage contains value like "152 / 152,682 / 682", thats two values separated by commas, I would like to divide first number by second number and replaces the value with the result
    def process_coverage(val):
        # Split by comma, process each "a / b" part, return comma-separated results
        parts = [v.strip() for v in str(val).split(",")]
        results = []
        for part in parts:
            if "/" in part:
                num, denom = part.split("/")
                try:
                    ratio = float(num.strip()) / float(denom.strip()) * 100
                    results.append(str(ratio))
                except Exception:
                    results.append(part)
            else:
                results.append(part)
        return ",".join(results)

    if convert_coverage:
        df.df["Query / Template length"] = df.df["Query / Template length"].apply(
            process_coverage
        )

    def extract_contig(val):
        # Extract the first part before any space or comma
        parts = [v.strip() for v in str(val).split(",")]
        results = []
        for part in parts:
            match = re.search(r"\b(\w+)\d+\b", part)
            if match:
                results.append(match.group(0))
            else:
                results.append(part)
        return ",".join(results)

    # If filter_contig is True, we look at the 'Contig' column, it contains "textXXXX more text,text2XXXXX more text", we want to extract textXXXX,text2XXXXX, etc
    if filter_contig:
        df.df["Contig"] = df.df["Contig"].apply(extract_contig)

    if filter_columns:
        df.filter_columns(filter_columns)

    if replace_header:
        df.rename_header(replace_header)

    # df.show()

    df.export_data(output_path, file_type="tsv")


@call_parse
def process_plasmidfinder_data_from_cli(
    input_path: str,
    output_path: str = "./output.tsv",
    replace_header: str = None,
    filter_columns: str = None,
    add_header: str = None,
    convert_coverage: bool = False,
    filter_contig: bool = False,
):
    process_plasmidfinder_data(
        input_path,
        output_path,
        replace_header,
        filter_columns,
        add_header,
        convert_coverage,
        filter_contig,
    )
